@@ -28,7 +28,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "logind.h"
 #include "kscreensaversettings.h"
 #include "powermanagement_inhibition.h"
+
+#include "kscreenlocker_logging.h"
 #include <config-kscreenlocker.h>
+
 #include <config-X11.h>
 #include "waylandserver.h"
 // KDE
@@ -38,10 +41,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <KNotification>
 #include <KGlobalAccel>
 
-//kwayland
-#include <KWayland/Server/display.h>
-#include <KWayland/Server/clientconnection.h>
-
 // Qt
 #include <QAction>
 #include <QKeyEvent>
@@ -83,7 +82,6 @@ KSldApp::KSldApp(QObject * parent)
     , m_lockProcess(nullptr)
     , m_lockWindow(nullptr)
     , m_waylandServer(new WaylandServer(this))
-    , m_waylandDisplay(nullptr)
     , m_lockedTimer(QElapsedTimer())
     , m_idleId(0)
     , m_lockGrace(0)
@@ -168,7 +166,7 @@ void KSldApp::initialize()
 
     // Global keys
     if (KAuthorized::authorizeAction(QStringLiteral("lock_screen"))) {
-        qDebug() << "Configuring Lock Action";
+        qCDebug(KSCREENLOCKER) << "Configuring Lock Action";
         QAction *a = new QAction(this);
         a->setObjectName(QStringLiteral("Lock Session"));
         a->setProperty("componentName", QStringLiteral("ksmserver"));
@@ -213,33 +211,56 @@ void KSldApp::initialize()
     auto finishedSignal = static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished);
     connect(m_lockProcess, finishedSignal, this,
         [this](int exitCode, QProcess::ExitStatus exitStatus) {
-            if (m_isWayland && m_waylandDisplay && m_greeterClientConnection) {
-                m_greeterClientConnection->destroy();
-            }
-            if ((!exitCode && exitStatus == QProcess::NormalExit) || s_graceTimeKill || s_logindExit) {
+            qCDebug(KSCREENLOCKER) << "Greeter process exitted with status:"
+                     << exitStatus << "exit code:" << exitCode;
+
+            const bool regularExit = !exitCode && exitStatus == QProcess::NormalExit;
+            if (regularExit || s_graceTimeKill || s_logindExit) {
                 // unlock process finished successfully - we can remove the lock grab
+
+                if (regularExit) {
+                    qCDebug(KSCREENLOCKER) << "Unlocking now on regular exit.";
+                } else if (s_graceTimeKill) {
+                    qCDebug(KSCREENLOCKER) << "Unlocking anyway due to grace time.";
+                } else {
+                    Q_ASSERT(s_logindExit);
+                    qCDebug(KSCREENLOCKER) << "Unlocking anyway since forced through logind.";
+                }
+
                 s_graceTimeKill = false;
                 s_logindExit = false;
                 doUnlock();
                 return;
             }
-            // failure, restart lock process
+
+            qCWarning(KSCREENLOCKER) << "Greeter process exit unregular. Restarting lock.";
+
             m_greeterCrashedCounter++;
             if (m_greeterCrashedCounter < 4) {
                 // Perhaps it crashed due to a graphics driver issue, force software rendering now
+                qCDebug(KSCREENLOCKER, "Trying to lock again with software rendering (%d/4).",
+                       m_greeterCrashedCounter);
                 setForceSoftwareRendering(true);
                 startLockProcess(EstablishLock::Immediate);
             } else if (m_lockWindow) {
+                qCWarning(KSCREENLOCKER)
+                        << "Everything else failed. Need to put Greeter in emergency mode.";
                 m_lockWindow->emergencyShow();
+            } else {
+                qCCritical(KSCREENLOCKER)
+                        << "Greeter process exitted and we could in no way recover from that!";
             }
         }
     );
     connect(m_lockProcess, &QProcess::errorOccurred, this,
         [this](QProcess::ProcessError error) {
             if (error == QProcess::FailedToStart) {
+                qCDebug(KSCREENLOCKER) << "Greeter Process  failed to start. Trying to directly unlock again.";
                 doUnlock();
                 m_waylandServer->stop();
-                qCritical() << "Greeter Process not available";
+                qCCritical(KSCREENLOCKER) << "Greeter Process not available";
+            } else {
+                qCWarning(KSCREENLOCKER) << "Greeter Process encountered an unhandled error:" << error;
             }
         }
     );
@@ -373,15 +394,15 @@ void KSldApp::lock(EstablishLock establishLock, int attemptCount)
         emit aboutToLock();
     }
 
-    qDebug() << "lock called";
+    qCDebug(KSCREENLOCKER) << "lock called";
     if (!establishGrab()) {
         if (attemptCount < 3) {
-            qWarning() << "Could not establish screen lock. Trying again in 10ms";
+            qCWarning(KSCREENLOCKER) << "Could not establish screen lock. Trying again in 10ms";
             QTimer::singleShot(10, this, [=]() {
                 lock(establishLock, attemptCount+1);
             });
         } else {
-            qCritical() << "Could not establish screen lock";
+            qCCritical(KSCREENLOCKER) << "Could not establish screen lock";
         }
         return;
     }
@@ -425,6 +446,9 @@ public:
 
 bool KSldApp::establishGrab()
 {
+    if (m_isWayland) {
+        return m_waylandFd >= 0;
+    }
     if (!m_isX11) {
         return true;
     }
@@ -513,7 +537,7 @@ static bool grabMouse()
 
 void KSldApp::doUnlock()
 {
-    qDebug() << "Grab Released";
+    qCDebug(KSCREENLOCKER) << "Grab Released";
     if (m_isX11) {
         xcb_connection_t *c = QX11Info::connection();
         xcb_ungrab_keyboard(c, XCB_CURRENT_TIME);
@@ -558,33 +582,22 @@ bool KSldApp::isFdoPowerInhibited() const
     return m_powerManagementInhibition->isInhibited();
 }
 
+void KSldApp::setWaylandFd(int fd)
+{
+    m_waylandFd = fd;
+}
+
 void KSldApp::startLockProcess(EstablishLock establishLock)
 {
     QProcessEnvironment env = m_greeterEnv;
 
-    if (m_isWayland && m_waylandDisplay) {
-        int sx[2];
-        if (socketpair(AF_UNIX, SOCK_STREAM | SOCK_CLOEXEC, 0, sx) < 0) {
-            qWarning() << "Can not create socket";
-            emit m_lockProcess->errorOccurred(QProcess::FailedToStart);
-            return;
-        }
-        m_greeterClientConnection = m_waylandDisplay->createClient(sx[0]);
-        connect(m_greeterClientConnection, &QObject::destroyed, this,
-            [this] (QObject *destroyedObject) {
-                if (destroyedObject != m_greeterClientConnection) {
-                    return;
-                }
-                m_greeterClientConnection = nullptr;
-                emit greeterClientConnectionChanged();
-            }
-        );
-        emit greeterClientConnectionChanged();
-        int socket = dup(sx[1]);
+    if (m_isWayland && m_waylandFd >= 0) {
+        int socket = dup(m_waylandFd);
         if (socket >= 0) {
             env.insert(QStringLiteral("WAYLAND_SOCKET"), QString::number(socket));
         }
     }
+
     QStringList args;
     if (establishLock == EstablishLock::Immediate) {
         args << QStringLiteral("--immediateLock");
@@ -608,6 +621,7 @@ void KSldApp::startLockProcess(EstablishLock establishLock)
     // start the Wayland server
     int fd = m_waylandServer->start();
     if (fd == -1) {
+        qCWarning(KSCREENLOCKER) << "Could not start the Wayland server.";
         emit m_lockProcess->errorOccurred(QProcess::FailedToStart);
         return;
     }
@@ -620,32 +634,46 @@ void KSldApp::startLockProcess(EstablishLock establishLock)
     close(fd);
 }
 
+void KSldApp::userActivity()
+{
+    if (isGraceTime()) {
+        unlock();
+    }
+    if (m_lockWindow) {
+        m_lockWindow->userActivity();
+    }
+}
+
 void KSldApp::showLockWindow()
 {
     if (!m_lockWindow) {
         if (m_isX11) {
             m_lockWindow = new X11Locker(this);
+
+            connect(m_lockWindow, &AbstractLocker::userActivity, m_lockWindow,
+                [this]() {
+                    if (isGraceTime()) {
+                        unlock();
+                    }
+                },
+                Qt::QueuedConnection
+            );
         }
+
         if (m_isWayland) {
-            m_lockWindow = new WaylandLocker(m_waylandDisplay, this);
+            m_lockWindow = new WaylandLocker(this);
         }
         if (!m_lockWindow) {
             return;
         }
         m_lockWindow->setGlobalAccel(m_globalAccel);
-        connect(m_lockWindow, &AbstractLocker::userActivity, m_lockWindow,
-            [this]() {
-                if (isGraceTime()) {
-                    unlock();
-                }
-            },
-            Qt::QueuedConnection
-        );
+
         connect(m_lockWindow, &AbstractLocker::lockWindowShown, m_lockWindow,
             [this] {
                 lockScreenShown();
             }
         , Qt::QueuedConnection);
+
         connect(m_waylandServer, &WaylandServer::x11WindowAdded, m_lockWindow, &AbstractLocker::addAllowedWindow);
     }
     m_lockWindow->showLockWindow();
@@ -711,13 +739,6 @@ void KSldApp::solidSuspend()
     }
 }
 
-void KSldApp::setWaylandDisplay(KWayland::Server::Display *display)
-{
-    if (m_waylandDisplay != display) {
-        m_waylandDisplay = display;
-    }
-}
-
 void KSldApp::lockScreenShown()
 {
     if (m_lockState == Locked) {
